@@ -31,6 +31,7 @@
 #include "src/core/ext/xds/xds_channel_args.h"
 #include "src/core/ext/xds/xds_client.h"
 #include "src/core/ext/xds/xds_http_filters.h"
+#include "src/core/ext/xds/xds_routing.h"
 #include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/iomgr/closure.h"
 #include "src/core/lib/iomgr/exec_ctx.h"
@@ -78,36 +79,39 @@ class XdsResolver : public Resolver {
   }
 
  private:
-  class Notifier {
-   public:
-    Notifier(RefCountedPtr<XdsResolver> resolver, XdsApi::LdsUpdate update);
-    Notifier(RefCountedPtr<XdsResolver> resolver, XdsApi::RdsUpdate update);
-    Notifier(RefCountedPtr<XdsResolver> resolver, grpc_error_handle error);
-    explicit Notifier(RefCountedPtr<XdsResolver> resolver);
-
-   private:
-    enum Type { kLdsUpdate, kRdsUpdate, kError, kDoesNotExist };
-
-    static void RunInExecCtx(void* arg, grpc_error_handle error);
-    void RunInWorkSerializer(grpc_error_handle error);
-
-    RefCountedPtr<XdsResolver> resolver_;
-    grpc_closure closure_;
-    XdsApi::LdsUpdate update_;
-    Type type_;
-  };
-
   class ListenerWatcher : public XdsClient::ListenerWatcherInterface {
    public:
     explicit ListenerWatcher(RefCountedPtr<XdsResolver> resolver)
         : resolver_(std::move(resolver)) {}
     void OnListenerChanged(XdsApi::LdsUpdate listener) override {
-      new Notifier(resolver_, std::move(listener));
+      Ref().release();  // ref held by lambda
+      resolver_->work_serializer_->Run(
+          // TODO(yashykt): When we move to C++14, capture listener with
+          // std::move
+          [this, listener]() mutable {
+            resolver_->OnListenerUpdate(std::move(listener));
+            Unref();
+          },
+          DEBUG_LOCATION);
     }
     void OnError(grpc_error_handle error) override {
-      new Notifier(resolver_, error);
+      Ref().release();  // ref held by lambda
+      resolver_->work_serializer_->Run(
+          [this, error]() {
+            resolver_->OnError(error);
+            Unref();
+          },
+          DEBUG_LOCATION);
+    }
+    void OnResourceDoesNotExist() override {
+      Ref().release();  // ref held by lambda
+      resolver_->work_serializer_->Run(
+          [this]() {
+            resolver_->OnResourceDoesNotExist();
+            Unref();
+          },
+          DEBUG_LOCATION);
     }
-    void OnResourceDoesNotExist() override { new Notifier(resolver_); }
 
    private:
     RefCountedPtr<XdsResolver> resolver_;
@@ -118,12 +122,34 @@ class XdsResolver : public Resolver {
     explicit RouteConfigWatcher(RefCountedPtr<XdsResolver> resolver)
         : resolver_(std::move(resolver)) {}
     void OnRouteConfigChanged(XdsApi::RdsUpdate route_config) override {
-      new Notifier(resolver_, std::move(route_config));
+      Ref().release();  // ref held by lambda
+      resolver_->work_serializer_->Run(
+          // TODO(yashykt): When we move to C++14, capture route_config with
+          // std::move
+          [this, route_config]() mutable {
+            resolver_->OnRouteConfigUpdate(std::move(route_config));
+            Unref();
+          },
+          DEBUG_LOCATION);
     }
     void OnError(grpc_error_handle error) override {
-      new Notifier(resolver_, error);
+      Ref().release();  // ref held by lambda
+      resolver_->work_serializer_->Run(
+          [this, error]() {
+            resolver_->OnError(error);
+            Unref();
+          },
+          DEBUG_LOCATION);
+    }
+    void OnResourceDoesNotExist() override {
+      Ref().release();  // ref held by lambda
+      resolver_->work_serializer_->Run(
+          [this]() {
+            resolver_->OnResourceDoesNotExist();
+            Unref();
+          },
+          DEBUG_LOCATION);
     }
-    void OnResourceDoesNotExist() override { new Notifier(resolver_); }
 
    private:
     RefCountedPtr<XdsResolver> resolver_;
@@ -232,10 +258,12 @@ class XdsResolver : public Resolver {
     };
     using RouteTable = std::vector<Route>;
 
+    class RouteListIterator;
+
     void MaybeAddCluster(const std::string& name);
     grpc_error_handle CreateMethodConfig(
         const XdsApi::Route& route,
-        const XdsApi::Route::ClusterWeight* cluster_weight,
+        const XdsApi::Route::RouteAction::ClusterWeight* cluster_weight,
         RefCountedPtr<ServiceConfig>* method_config);
 
     RefCountedPtr<XdsResolver> resolver_;
@@ -275,71 +303,6 @@ class XdsResolver : public Resolver {
   ClusterState::ClusterStateMap cluster_state_map_;
 };
 
-//
-// XdsResolver::Notifier
-//
-
-XdsResolver::Notifier::Notifier(RefCountedPtr<XdsResolver> resolver,
-                                XdsApi::LdsUpdate update)
-    : resolver_(std::move(resolver)),
-      update_(std::move(update)),
-      type_(kLdsUpdate) {
-  GRPC_CLOSURE_INIT(&closure_, &RunInExecCtx, this, nullptr);
-  ExecCtx::Run(DEBUG_LOCATION, &closure_, GRPC_ERROR_NONE);
-}
-
-XdsResolver::Notifier::Notifier(RefCountedPtr<XdsResolver> resolver,
-                                XdsApi::RdsUpdate update)
-    : resolver_(std::move(resolver)), type_(kRdsUpdate) {
-  update_.http_connection_manager.rds_update = std::move(update);
-  GRPC_CLOSURE_INIT(&closure_, &RunInExecCtx, this, nullptr);
-  ExecCtx::Run(DEBUG_LOCATION, &closure_, GRPC_ERROR_NONE);
-}
-
-XdsResolver::Notifier::Notifier(RefCountedPtr<XdsResolver> resolver,
-                                grpc_error_handle error)
-    : resolver_(std::move(resolver)), type_(kError) {
-  GRPC_CLOSURE_INIT(&closure_, &RunInExecCtx, this, nullptr);
-  ExecCtx::Run(DEBUG_LOCATION, &closure_, error);
-}
-
-XdsResolver::Notifier::Notifier(RefCountedPtr<XdsResolver> resolver)
-    : resolver_(std::move(resolver)), type_(kDoesNotExist) {
-  GRPC_CLOSURE_INIT(&closure_, &RunInExecCtx, this, nullptr);
-  ExecCtx::Run(DEBUG_LOCATION, &closure_, GRPC_ERROR_NONE);
-}
-
-void XdsResolver::Notifier::RunInExecCtx(void* arg, grpc_error_handle error) {
-  Notifier* self = static_cast<Notifier*>(arg);
-  GRPC_ERROR_REF(error);
-  self->resolver_->work_serializer_->Run(
-      [self, error]() { self->RunInWorkSerializer(error); }, DEBUG_LOCATION);
-}
-
-void XdsResolver::Notifier::RunInWorkSerializer(grpc_error_handle error) {
-  if (resolver_->xds_client_ == nullptr) {
-    GRPC_ERROR_UNREF(error);
-    delete this;
-    return;
-  }
-  switch (type_) {
-    case kLdsUpdate:
-      resolver_->OnListenerUpdate(std::move(update_));
-      break;
-    case kRdsUpdate:
-      resolver_->OnRouteConfigUpdate(
-          std::move(*update_.http_connection_manager.rds_update));
-      break;
-    case kError:
-      resolver_->OnError(error);
-      break;
-    case kDoesNotExist:
-      resolver_->OnResourceDoesNotExist();
-      break;
-  };
-  delete this;
-}
-
 //
 // XdsResolver::XdsConfigSelector::Route
 //
@@ -363,6 +326,25 @@ bool XdsResolver::XdsConfigSelector::Route::operator==(
          MethodConfigsEqual(method_config.get(), other.method_config.get());
 }
 
+// Implementation of XdsRouting::RouteListIterator for getting the matching
+// route for a request.
+class XdsResolver::XdsConfigSelector::RouteListIterator
+    : public XdsRouting::RouteListIterator {
+ public:
+  explicit RouteListIterator(const RouteTable* route_table)
+      : route_table_(route_table) {}
+
+  size_t Size() const override { return route_table_->size(); }
+
+  const XdsApi::Route::Matchers& GetMatchersForRoute(
+      size_t index) const override {
+    return (*route_table_)[index].route.matchers;
+  }
+
+ private:
+  const RouteTable* route_table_;
+};
+
 //
 // XdsResolver::XdsConfigSelector
 //
@@ -392,30 +374,34 @@ XdsResolver::XdsConfigSelector::XdsConfigSelector(
     route_table_.emplace_back();
     auto& route_entry = route_table_.back();
     route_entry.route = route;
-    // If the route doesn't specify a timeout, set its timeout to the global
-    // one.
-    if (!route.max_stream_duration.has_value()) {
-      route_entry.route.max_stream_duration =
-          resolver_->current_listener_.http_connection_manager
-              .http_max_stream_duration;
-    }
-    if (route.weighted_clusters.empty()) {
-      *error = CreateMethodConfig(route_entry.route, nullptr,
-                                  &route_entry.method_config);
-      MaybeAddCluster(route.cluster_name);
-    } else {
-      uint32_t end = 0;
-      for (const auto& weighted_cluster : route_entry.route.weighted_clusters) {
-        Route::ClusterWeightState cluster_weight_state;
-        *error = CreateMethodConfig(route_entry.route, &weighted_cluster,
-                                    &cluster_weight_state.method_config);
-        if (*error != GRPC_ERROR_NONE) return;
-        end += weighted_cluster.weight;
-        cluster_weight_state.range_end = end;
-        cluster_weight_state.cluster = weighted_cluster.name;
-        route_entry.weighted_cluster_state.push_back(
-            std::move(cluster_weight_state));
-        MaybeAddCluster(weighted_cluster.name);
+    auto* route_action =
+        absl::get_if<XdsApi::Route::RouteAction>(&route_entry.route.action);
+    if (route_action != nullptr) {
+      // If the route doesn't specify a timeout, set its timeout to the global
+      // one.
+      if (!route_action->max_stream_duration.has_value()) {
+        route_action->max_stream_duration =
+            resolver_->current_listener_.http_connection_manager
+                .http_max_stream_duration;
+      }
+      if (route_action->weighted_clusters.empty()) {
+        *error = CreateMethodConfig(route_entry.route, nullptr,
+                                    &route_entry.method_config);
+        MaybeAddCluster(route_action->cluster_name);
+      } else {
+        uint32_t end = 0;
+        for (const auto& weighted_cluster : route_action->weighted_clusters) {
+          Route::ClusterWeightState cluster_weight_state;
+          *error = CreateMethodConfig(route_entry.route, &weighted_cluster,
+                                      &cluster_weight_state.method_config);
+          if (*error != GRPC_ERROR_NONE) return;
+          end += weighted_cluster.weight;
+          cluster_weight_state.range_end = end;
+          cluster_weight_state.cluster = weighted_cluster.name;
+          route_entry.weighted_cluster_state.push_back(
+              std::move(cluster_weight_state));
+          MaybeAddCluster(weighted_cluster.name);
+        }
       }
     }
   }
@@ -444,32 +430,16 @@ XdsResolver::XdsConfigSelector::~XdsConfigSelector() {
   resolver_->MaybeRemoveUnusedClusters();
 }
 
-const XdsHttpFilterImpl::FilterConfig* FindFilterConfigOverride(
-    const std::string& instance_name,
-    const XdsApi::RdsUpdate::VirtualHost& vhost, const XdsApi::Route& route,
-    const XdsApi::Route::ClusterWeight* cluster_weight) {
-  // Check ClusterWeight, if any.
-  if (cluster_weight != nullptr) {
-    auto it = cluster_weight->typed_per_filter_config.find(instance_name);
-    if (it != cluster_weight->typed_per_filter_config.end()) return &it->second;
-  }
-  // Check Route.
-  auto it = route.typed_per_filter_config.find(instance_name);
-  if (it != route.typed_per_filter_config.end()) return &it->second;
-  // Check VirtualHost.
-  it = vhost.typed_per_filter_config.find(instance_name);
-  if (it != vhost.typed_per_filter_config.end()) return &it->second;
-  // Not found.
-  return nullptr;
-}
-
 grpc_error_handle XdsResolver::XdsConfigSelector::CreateMethodConfig(
     const XdsApi::Route& route,
-    const XdsApi::Route::ClusterWeight* cluster_weight,
+    const XdsApi::Route::RouteAction::ClusterWeight* cluster_weight,
     RefCountedPtr<ServiceConfig>* method_config) {
   std::vector<std::string> fields;
+  const auto& route_action =
+      absl::get<XdsApi::Route::RouteAction>(route.action);
   // Set retry policy if any.
-  if (route.retry_policy.has_value()) {
+  if (route_action.retry_policy.has_value() &&
+      !route_action.retry_policy->retry_on.Empty()) {
     std::vector<std::string> retry_parts;
     retry_parts.push_back(absl::StrFormat(
         "\"retryPolicy\": {\n"
@@ -477,25 +447,27 @@ grpc_error_handle XdsResolver::XdsConfigSelector::CreateMethodConfig(
         "      \"initialBackoff\": \"%d.%09ds\",\n"
         "      \"maxBackoff\": \"%d.%09ds\",\n"
         "      \"backoffMultiplier\": 2,\n",
-        route.retry_policy->num_retries + 1,
-        route.retry_policy->retry_back_off.base_interval.seconds,
-        route.retry_policy->retry_back_off.base_interval.nanos,
-        route.retry_policy->retry_back_off.max_interval.seconds,
-        route.retry_policy->retry_back_off.max_interval.nanos));
+        route_action.retry_policy->num_retries + 1,
+        route_action.retry_policy->retry_back_off.base_interval.seconds,
+        route_action.retry_policy->retry_back_off.base_interval.nanos,
+        route_action.retry_policy->retry_back_off.max_interval.seconds,
+        route_action.retry_policy->retry_back_off.max_interval.nanos));
     std::vector<std::string> code_parts;
-    if (route.retry_policy->retry_on.Contains(GRPC_STATUS_CANCELLED)) {
+    if (route_action.retry_policy->retry_on.Contains(GRPC_STATUS_CANCELLED)) {
       code_parts.push_back("        \"CANCELLED\"");
     }
-    if (route.retry_policy->retry_on.Contains(GRPC_STATUS_DEADLINE_EXCEEDED)) {
+    if (route_action.retry_policy->retry_on.Contains(
+            GRPC_STATUS_DEADLINE_EXCEEDED)) {
       code_parts.push_back("        \"DEADLINE_EXCEEDED\"");
     }
-    if (route.retry_policy->retry_on.Contains(GRPC_STATUS_INTERNAL)) {
+    if (route_action.retry_policy->retry_on.Contains(GRPC_STATUS_INTERNAL)) {
       code_parts.push_back("        \"INTERNAL\"");
     }
-    if (route.retry_policy->retry_on.Contains(GRPC_STATUS_RESOURCE_EXHAUSTED)) {
+    if (route_action.retry_policy->retry_on.Contains(
+            GRPC_STATUS_RESOURCE_EXHAUSTED)) {
       code_parts.push_back("        \"RESOURCE_EXHAUSTED\"");
     }
-    if (route.retry_policy->retry_on.Contains(GRPC_STATUS_UNAVAILABLE)) {
+    if (route_action.retry_policy->retry_on.Contains(GRPC_STATUS_UNAVAILABLE)) {
       code_parts.push_back("        \"UNAVAILABLE\"");
     }
     retry_parts.push_back(
@@ -505,49 +477,24 @@ grpc_error_handle XdsResolver::XdsConfigSelector::CreateMethodConfig(
     fields.emplace_back(absl::StrJoin(retry_parts, ""));
   }
   // Set timeout.
-  if (route.max_stream_duration.has_value() &&
-      (route.max_stream_duration->seconds != 0 ||
-       route.max_stream_duration->nanos != 0)) {
-    fields.emplace_back(absl::StrFormat("    \"timeout\": \"%d.%09ds\"",
-                                        route.max_stream_duration->seconds,
-                                        route.max_stream_duration->nanos));
+  if (route_action.max_stream_duration.has_value() &&
+      (route_action.max_stream_duration->seconds != 0 ||
+       route_action.max_stream_duration->nanos != 0)) {
+    fields.emplace_back(
+        absl::StrFormat("    \"timeout\": \"%d.%09ds\"",
+                        route_action.max_stream_duration->seconds,
+                        route_action.max_stream_duration->nanos));
   }
   // Handle xDS HTTP filters.
-  std::map<std::string, std::vector<std::string>> per_filter_configs;
-  grpc_channel_args* args = grpc_channel_args_copy(resolver_->args_);
-  for (const auto& http_filter :
-       resolver_->current_listener_.http_connection_manager.http_filters) {
-    // Find filter.  This is guaranteed to succeed, because it's checked
-    // at config validation time in the XdsApi code.
-    const XdsHttpFilterImpl* filter_impl =
-        XdsHttpFilterRegistry::GetFilterForType(
-            http_filter.config.config_proto_type_name);
-    GPR_ASSERT(filter_impl != nullptr);
-    // If there is not actually any C-core filter associated with this
-    // xDS filter, then it won't need any config, so skip it.
-    if (filter_impl->channel_filter() == nullptr) continue;
-    // Allow filter to add channel args that may affect service config
-    // parsing.
-    args = filter_impl->ModifyChannelArgs(args);
-    // Find config override, if any.
-    const XdsHttpFilterImpl::FilterConfig* config_override =
-        FindFilterConfigOverride(http_filter.name,
-                                 resolver_->current_virtual_host_, route,
-                                 cluster_weight);
-    // Generate service config for filter.
-    auto method_config_field =
-        filter_impl->GenerateServiceConfig(http_filter.config, config_override);
-    if (!method_config_field.ok()) {
-      return GRPC_ERROR_CREATE_FROM_COPIED_STRING(
-          absl::StrCat("failed to generate method config for HTTP filter ",
-                       http_filter.name, ": ",
-                       method_config_field.status().ToString())
-              .c_str());
-    }
-    per_filter_configs[method_config_field->service_config_field_name]
-        .push_back(method_config_field->element);
+  XdsRouting::GeneratePerHttpFilterConfigsResult result =
+      XdsRouting::GeneratePerHTTPFilterConfigs(
+          resolver_->current_listener_.http_connection_manager.http_filters,
+          resolver_->current_virtual_host_, route, cluster_weight,
+          grpc_channel_args_copy(resolver_->args_));
+  if (result.error != GRPC_ERROR_NONE) {
+    return result.error;
   }
-  for (const auto& p : per_filter_configs) {
+  for (const auto& p : result.per_filter_configs) {
     fields.emplace_back(absl::StrCat("    \"", p.first, "\": [\n",
                                      absl::StrJoin(p.second, ",\n"),
                                      "\n    ]"));
@@ -565,9 +512,9 @@ grpc_error_handle XdsResolver::XdsConfigSelector::CreateMethodConfig(
         absl::StrJoin(fields, ",\n"),
         "\n  } ]\n"
         "}");
-    *method_config = ServiceConfig::Create(args, json.c_str(), &error);
+    *method_config = ServiceConfig::Create(result.args, json.c_str(), &error);
   }
-  grpc_channel_args_destroy(args);
+  grpc_channel_args_destroy(result.args);
   return error;
 }
 
@@ -588,40 +535,13 @@ void XdsResolver::XdsConfigSelector::MaybeAddCluster(const std::string& name) {
   }
 }
 
-absl::optional<absl::string_view> GetHeaderValue(
-    grpc_metadata_batch* initial_metadata, absl::string_view header_name,
-    std::string* concatenated_value) {
-  // Note: If we ever allow binary headers here, we still need to
-  // special-case ignore "grpc-tags-bin" and "grpc-trace-bin", since
-  // they are not visible to the LB policy in grpc-go.
-  if (absl::EndsWith(header_name, "-bin")) {
-    return absl::nullopt;
-  } else if (header_name == "content-type") {
-    return "application/grpc";
-  }
-  return grpc_metadata_batch_get_value(initial_metadata, header_name,
-                                       concatenated_value);
-}
-
-bool HeadersMatch(const std::vector<HeaderMatcher>& header_matchers,
-                  grpc_metadata_batch* initial_metadata) {
-  for (const auto& header_matcher : header_matchers) {
-    std::string concatenated_value;
-    if (!header_matcher.Match(GetHeaderValue(
-            initial_metadata, header_matcher.name(), &concatenated_value))) {
-      return false;
-    }
-  }
-  return true;
-}
-
 absl::optional<uint64_t> HeaderHashHelper(
-    const XdsApi::Route::HashPolicy& policy,
+    const XdsApi::Route::RouteAction::HashPolicy& policy,
     grpc_metadata_batch* initial_metadata) {
-  GPR_ASSERT(policy.type == XdsApi::Route::HashPolicy::HEADER);
+  GPR_ASSERT(policy.type == XdsApi::Route::RouteAction::HashPolicy::HEADER);
   std::string value_buffer;
-  absl::optional<absl::string_view> header_value =
-      GetHeaderValue(initial_metadata, policy.header_name, &value_buffer);
+  absl::optional<absl::string_view> header_value = XdsRouting::GetHeaderValue(
+      initial_metadata, policy.header_name, &value_buffer);
   if (!header_value.has_value()) {
     return absl::nullopt;
   }
@@ -637,119 +557,112 @@ absl::optional<uint64_t> HeaderHashHelper(
   return XXH64(header_value->data(), header_value->size(), 0);
 }
 
-bool UnderFraction(const uint32_t fraction_per_million) {
-  // Generate a random number in [0, 1000000).
-  const uint32_t random_number = rand() % 1000000;
-  return random_number < fraction_per_million;
-}
-
 ConfigSelector::CallConfig XdsResolver::XdsConfigSelector::GetCallConfig(
     GetCallConfigArgs args) {
-  for (const auto& entry : route_table_) {
-    // Path matching.
-    if (!entry.route.matchers.path_matcher.Match(
-            StringViewFromSlice(*args.path))) {
-      continue;
-    }
-    // Header Matching.
-    if (!HeadersMatch(entry.route.matchers.header_matchers,
-                      args.initial_metadata)) {
-      continue;
-    }
-    // Match fraction check
-    if (entry.route.matchers.fraction_per_million.has_value() &&
-        !UnderFraction(entry.route.matchers.fraction_per_million.value())) {
-      continue;
-    }
-    // Found a route match
-    absl::string_view cluster_name;
-    RefCountedPtr<ServiceConfig> method_config;
-    if (entry.route.weighted_clusters.empty()) {
-      cluster_name = entry.route.cluster_name;
-      method_config = entry.method_config;
-    } else {
-      const uint32_t key =
-          rand() %
-          entry.weighted_cluster_state[entry.weighted_cluster_state.size() - 1]
-              .range_end;
-      // Find the index in weighted clusters corresponding to key.
-      size_t mid = 0;
-      size_t start_index = 0;
-      size_t end_index = entry.weighted_cluster_state.size() - 1;
-      size_t index = 0;
-      while (end_index > start_index) {
-        mid = (start_index + end_index) / 2;
-        if (entry.weighted_cluster_state[mid].range_end > key) {
-          end_index = mid;
-        } else if (entry.weighted_cluster_state[mid].range_end < key) {
-          start_index = mid + 1;
-        } else {
-          index = mid + 1;
-          break;
-        }
-      }
-      if (index == 0) index = start_index;
-      GPR_ASSERT(entry.weighted_cluster_state[index].range_end > key);
-      cluster_name = entry.weighted_cluster_state[index].cluster;
-      method_config = entry.weighted_cluster_state[index].method_config;
-    }
-    auto it = clusters_.find(cluster_name);
-    GPR_ASSERT(it != clusters_.end());
-    // Generate a hash.
-    absl::optional<uint64_t> hash;
-    for (const auto& hash_policy : entry.route.hash_policies) {
-      absl::optional<uint64_t> new_hash;
-      switch (hash_policy.type) {
-        case XdsApi::Route::HashPolicy::HEADER:
-          new_hash = HeaderHashHelper(hash_policy, args.initial_metadata);
-          break;
-        case XdsApi::Route::HashPolicy::CHANNEL_ID:
-          new_hash = static_cast<uint64_t>(
-              reinterpret_cast<uintptr_t>(resolver_.get()));
-          break;
-        default:
-          GPR_ASSERT(0);
-      }
-      if (new_hash.has_value()) {
-        // Rotating the old value prevents duplicate hash rules from cancelling
-        // each other out and preserves all of the entropy
-        const uint64_t old_value =
-            hash.has_value() ? ((hash.value() << 1) | (hash.value() >> 63)) : 0;
-        hash = old_value ^ new_hash.value();
-      }
-      // If the policy is a terminal policy and a hash has been generated,
-      // ignore the rest of the hash policies.
-      if (hash_policy.terminal && hash.has_value()) {
+  auto route_index = XdsRouting::GetRouteForRequest(
+      RouteListIterator(&route_table_), StringViewFromSlice(*args.path),
+      args.initial_metadata);
+  if (!route_index.has_value()) {
+    return CallConfig();
+  }
+  auto& entry = route_table_[*route_index];
+  // Found a route match
+  const auto* route_action =
+      absl::get_if<XdsApi::Route::RouteAction>(&entry.route.action);
+  if (route_action == nullptr) {
+    CallConfig call_config;
+    call_config.error =
+        grpc_error_set_int(GRPC_ERROR_CREATE_FROM_STATIC_STRING(
+                               "Matching route has inappropriate action"),
+                           GRPC_ERROR_INT_GRPC_STATUS, GRPC_STATUS_UNAVAILABLE);
+    return call_config;
+  }
+  absl::string_view cluster_name;
+  RefCountedPtr<ServiceConfig> method_config;
+  if (route_action->weighted_clusters.empty()) {
+    cluster_name = route_action->cluster_name;
+    method_config = entry.method_config;
+  } else {
+    const uint32_t key =
+        rand() %
+        entry.weighted_cluster_state[entry.weighted_cluster_state.size() - 1]
+            .range_end;
+    // Find the index in weighted clusters corresponding to key.
+    size_t mid = 0;
+    size_t start_index = 0;
+    size_t end_index = entry.weighted_cluster_state.size() - 1;
+    size_t index = 0;
+    while (end_index > start_index) {
+      mid = (start_index + end_index) / 2;
+      if (entry.weighted_cluster_state[mid].range_end > key) {
+        end_index = mid;
+      } else if (entry.weighted_cluster_state[mid].range_end < key) {
+        start_index = mid + 1;
+      } else {
+        index = mid + 1;
         break;
       }
     }
-    if (!hash.has_value()) {
-      // If there is no hash, we just choose a random value as a default.
-      // We cannot directly use the result of rand() as the hash value,
-      // since it is a 32-bit number and not a 64-bit number and will
-      // therefore not be evenly distributed.
-      uint32_t upper = rand();
-      uint32_t lower = rand();
-      hash = (static_cast<uint64_t>(upper) << 32) | lower;
-    }
-    CallConfig call_config;
-    if (method_config != nullptr) {
-      call_config.method_configs =
-          method_config->GetMethodParsedConfigVector(grpc_empty_slice());
-      call_config.service_config = std::move(method_config);
-    }
-    call_config.call_attributes[kXdsClusterAttribute] = it->first;
-    std::string hash_string = absl::StrCat(hash.value());
-    char* hash_value =
-        static_cast<char*>(args.arena->Alloc(hash_string.size() + 1));
-    memcpy(hash_value, hash_string.c_str(), hash_string.size());
-    hash_value[hash_string.size()] = '\0';
-    call_config.call_attributes[kRequestRingHashAttribute] = hash_value;
-    call_config.call_dispatch_controller =
-        args.arena->New<XdsCallDispatchController>(it->second->Ref());
-    return call_config;
+    if (index == 0) index = start_index;
+    GPR_ASSERT(entry.weighted_cluster_state[index].range_end > key);
+    cluster_name = entry.weighted_cluster_state[index].cluster;
+    method_config = entry.weighted_cluster_state[index].method_config;
   }
-  return CallConfig();
+  auto it = clusters_.find(cluster_name);
+  GPR_ASSERT(it != clusters_.end());
+  // Generate a hash.
+  absl::optional<uint64_t> hash;
+  for (const auto& hash_policy : route_action->hash_policies) {
+    absl::optional<uint64_t> new_hash;
+    switch (hash_policy.type) {
+      case XdsApi::Route::RouteAction::HashPolicy::HEADER:
+        new_hash = HeaderHashHelper(hash_policy, args.initial_metadata);
+        break;
+      case XdsApi::Route::RouteAction::HashPolicy::CHANNEL_ID:
+        new_hash =
+            static_cast<uint64_t>(reinterpret_cast<uintptr_t>(resolver_.get()));
+        break;
+      default:
+        GPR_ASSERT(0);
+    }
+    if (new_hash.has_value()) {
+      // Rotating the old value prevents duplicate hash rules from cancelling
+      // each other out and preserves all of the entropy
+      const uint64_t old_value =
+          hash.has_value() ? ((hash.value() << 1) | (hash.value() >> 63)) : 0;
+      hash = old_value ^ new_hash.value();
+    }
+    // If the policy is a terminal policy and a hash has been generated,
+    // ignore the rest of the hash policies.
+    if (hash_policy.terminal && hash.has_value()) {
+      break;
+    }
+  }
+  if (!hash.has_value()) {
+    // If there is no hash, we just choose a random value as a default.
+    // We cannot directly use the result of rand() as the hash value,
+    // since it is a 32-bit number and not a 64-bit number and will
+    // therefore not be evenly distributed.
+    uint32_t upper = rand();
+    uint32_t lower = rand();
+    hash = (static_cast<uint64_t>(upper) << 32) | lower;
+  }
+  CallConfig call_config;
+  if (method_config != nullptr) {
+    call_config.method_configs =
+        method_config->GetMethodParsedConfigVector(grpc_empty_slice());
+    call_config.service_config = std::move(method_config);
+  }
+  call_config.call_attributes[kXdsClusterAttribute] = it->first;
+  std::string hash_string = absl::StrCat(hash.value());
+  char* hash_value =
+      static_cast<char*>(args.arena->Alloc(hash_string.size() + 1));
+  memcpy(hash_value, hash_string.c_str(), hash_string.size());
+  hash_value[hash_string.size()] = '\0';
+  call_config.call_attributes[kRequestRingHashAttribute] = hash_value;
+  call_config.call_dispatch_controller =
+      args.arena->New<XdsCallDispatchController>(it->second->Ref());
+  return call_config;
 }
 
 //
@@ -769,13 +682,7 @@ void XdsResolver::StartLocked() {
   }
   grpc_pollset_set_add_pollset_set(xds_client_->interested_parties(),
                                    interested_parties_);
-  channelz::ChannelNode* parent_channelz_node =
-      grpc_channel_args_find_pointer<channelz::ChannelNode>(
-          args_, GRPC_ARG_CHANNELZ_CHANNEL_NODE);
-  if (parent_channelz_node != nullptr) {
-    xds_client_->AddChannelzLinkage(parent_channelz_node);
-  }
-  auto watcher = absl::make_unique<ListenerWatcher>(Ref());
+  auto watcher = MakeRefCounted<ListenerWatcher>(Ref());
   listener_watcher_ = watcher.get();
   xds_client_->WatchListenerData(server_name_, std::move(watcher));
 }
@@ -793,12 +700,6 @@ void XdsResolver::ShutdownLocked() {
       xds_client_->CancelRouteConfigDataWatch(
           server_name_, route_config_watcher_, /*delay_unsubscription=*/false);
     }
-    channelz::ChannelNode* parent_channelz_node =
-        grpc_channel_args_find_pointer<channelz::ChannelNode>(
-            args_, GRPC_ARG_CHANNELZ_CHANNEL_NODE);
-    if (parent_channelz_node != nullptr) {
-      xds_client_->RemoveChannelzLinkage(parent_channelz_node);
-    }
     grpc_pollset_set_del_pollset_set(xds_client_->interested_parties(),
                                      interested_parties_);
     xds_client_.reset();
@@ -809,6 +710,9 @@ void XdsResolver::OnListenerUpdate(XdsApi::LdsUpdate listener) {
   if (GRPC_TRACE_FLAG_ENABLED(grpc_xds_resolver_trace)) {
     gpr_log(GPR_INFO, "[xds_resolver %p] received updated listener data", this);
   }
+  if (xds_client_ == nullptr) {
+    return;
+  }
   if (listener.http_connection_manager.route_config_name !=
       route_config_name_) {
     if (route_config_watcher_ != nullptr) {
@@ -822,7 +726,7 @@ void XdsResolver::OnListenerUpdate(XdsApi::LdsUpdate listener) {
         std::move(listener.http_connection_manager.route_config_name);
     if (!route_config_name_.empty()) {
       current_virtual_host_.routes.clear();
-      auto watcher = absl::make_unique<RouteConfigWatcher>(Ref());
+      auto watcher = MakeRefCounted<RouteConfigWatcher>(Ref());
       route_config_watcher_ = watcher.get();
       xds_client_->WatchRouteConfigData(route_config_name_, std::move(watcher));
     }
@@ -840,22 +744,43 @@ void XdsResolver::OnListenerUpdate(XdsApi::LdsUpdate listener) {
   }
 }
 
+namespace {
+class VirtualHostListIterator : public XdsRouting::VirtualHostListIterator {
+ public:
+  explicit VirtualHostListIterator(
+      const std::vector<XdsApi::RdsUpdate::VirtualHost>* virtual_hosts)
+      : virtual_hosts_(virtual_hosts) {}
+
+  size_t Size() const override { return virtual_hosts_->size(); }
+
+  const std::vector<std::string>& GetDomainsForVirtualHost(
+      size_t index) const override {
+    return (*virtual_hosts_)[index].domains;
+  }
+
+ private:
+  const std::vector<XdsApi::RdsUpdate::VirtualHost>* virtual_hosts_;
+};
+}  // namespace
+
 void XdsResolver::OnRouteConfigUpdate(XdsApi::RdsUpdate rds_update) {
   if (GRPC_TRACE_FLAG_ENABLED(grpc_xds_resolver_trace)) {
     gpr_log(GPR_INFO, "[xds_resolver %p] received updated route config", this);
   }
+  if (xds_client_ == nullptr) {
+    return;
+  }
   // Find the relevant VirtualHost from the RouteConfiguration.
-  XdsApi::RdsUpdate::VirtualHost* vhost =
-      rds_update.FindVirtualHostForDomain(server_name_);
-  if (vhost == nullptr) {
-    OnError(GRPC_ERROR_CREATE_FROM_COPIED_STRING(
+  auto vhost_index = XdsRouting::FindVirtualHostForDomain(
+      VirtualHostListIterator(&rds_update.virtual_hosts), server_name_);
+  if (!vhost_index.has_value()) {
+    OnError(GRPC_ERROR_CREATE_FROM_CPP_STRING(
         absl::StrCat("could not find VirtualHost for ", server_name_,
-                     " in RouteConfiguration")
-            .c_str()));
+                     " in RouteConfiguration")));
     return;
   }
   // Save the virtual host in the resolver.
-  current_virtual_host_ = std::move(*vhost);
+  current_virtual_host_ = std::move(rds_update.virtual_hosts[*vhost_index]);
   // Send a new result to the channel.
   GenerateResult();
 }
@@ -863,6 +788,10 @@ void XdsResolver::OnRouteConfigUpdate(XdsApi::RdsUpdate rds_update) {
 void XdsResolver::OnError(grpc_error_handle error) {
   gpr_log(GPR_ERROR, "[xds_resolver %p] received error from XdsClient: %s",
           this, grpc_error_std_string(error).c_str());
+  if (xds_client_ == nullptr) {
+    GRPC_ERROR_UNREF(error);
+    return;
+  }
   Result result;
   grpc_arg new_arg = xds_client_->MakeChannelArg();
   result.args = grpc_channel_args_copy_and_add(args_, &new_arg, 1);
@@ -875,6 +804,9 @@ void XdsResolver::OnResourceDoesNotExist() {
           "[xds_resolver %p] LDS/RDS resource does not exist -- clearing "
           "update and returning empty service config",
           this);
+  if (xds_client_ == nullptr) {
+    return;
+  }
   current_virtual_host_.routes.clear();
   Result result;
   result.service_config =
@@ -923,13 +855,15 @@ void XdsResolver::GenerateResult() {
   grpc_error_handle error = GRPC_ERROR_NONE;
   auto config_selector = MakeRefCounted<XdsConfigSelector>(Ref(), &error);
   if (error != GRPC_ERROR_NONE) {
-    OnError(error);
+    OnError(grpc_error_set_int(error, GRPC_ERROR_INT_GRPC_STATUS,
+                               GRPC_STATUS_UNAVAILABLE));
     return;
   }
   Result result;
   error = CreateServiceConfig(&result.service_config);
   if (error != GRPC_ERROR_NONE) {
-    OnError(error);
+    OnError(grpc_error_set_int(error, GRPC_ERROR_INT_GRPC_STATUS,
+                               GRPC_STATUS_UNAVAILABLE));
     return;
   }
   if (GRPC_TRACE_FLAG_ENABLED(grpc_xds_resolver_trace)) {
