@@ -14,10 +14,11 @@
 
 #include <grpc/support/port_platform.h>
 
+#include "src/core/lib/security/authorization/matchers.h"
+
 #include <grpc/grpc_security_constants.h>
 
 #include "src/core/lib/address_utils/sockaddr_utils.h"
-#include "src/core/lib/security/authorization/matchers.h"
 
 namespace grpc_core {
 
@@ -174,7 +175,8 @@ bool PortAuthorizationMatcher::Matches(const EvaluateArgs& args) const {
 
 bool AuthenticatedAuthorizationMatcher::Matches(
     const EvaluateArgs& args) const {
-  if (args.GetTransportSecurityType() != GRPC_SSL_TRANSPORT_SECURITY_TYPE) {
+  if (args.GetTransportSecurityType() != GRPC_SSL_TRANSPORT_SECURITY_TYPE &&
+      args.GetTransportSecurityType() != GRPC_TLS_TRANSPORT_SECURITY_TYPE) {
     // Connection is not authenticated.
     return false;
   }
@@ -198,8 +200,7 @@ bool AuthenticatedAuthorizationMatcher::Matches(
       }
     }
   }
-  // TODO(ashithasantosh): Check Subject field from certificate.
-  return false;
+  return matcher_.Match(args.GetSubject());
 }
 
 bool ReqServerNameAuthorizationMatcher::Matches(const EvaluateArgs&) const {
